searchNodes=[{"doc":"Surface is component based library for Phoenix LiveView. Built on top of the new Phoenix.LiveComponent API, Surface provides a more declarative way to express and use components in Phoenix. A work-in-progress live demo with more details can be found at surface-demo.msaraiva.io This module defines the ~H sigil that should be used to translate Surface code into Phoenix templates. In order to have ~H available for any Phoenix view, add the following import to your web file in lib/my_app_web.ex: # lib/my_app_web.ex ... def view do quote do ... import Surface end end Defining components To create a component you need to define a module and use one of the available component types: Surface.Component - A stateless component. Surface.LiveComponent - A live stateful component. Surface.LiveView - A wrapper component around Phoenix.LiveView. Surface.DataComponent - A component that serves as a customizable data holder for the parent component. Surface.MacroComponent - A low-level component which is responsible for translating its own content at compile time. Example # A functional stateless component defmodule Button do use Surface.Component property click, :event property kind, :string, default: &quot;is-info&quot; def render(assigns) do ~H&quot;&quot;&quot; &lt;button class=&quot;button {{ @kind }}&quot; phx-click={{ @click }}&gt; {{ @inner_content.() }} &lt;/button&gt; &quot;&quot;&quot; end end You can visit the documentation of each type of component for further explanation and examples. Directives Directives are built-in attributes that can modify the translated code of a component at compile time. Currently, the following directives are supported: :for - Iterates over a list (generator) and renders the content of the tag (or component) for each item in the list. :if - Conditionally render a tag (or component). The code will be rendered if the expression is evaluated to a truthy value. :show - Conditionally shows/hides an HTML tag, keeping the rendered alement in the DOM even when the value is false. :bindings - Defines the name of the variables (bindings) in the current scope that represent the values passed internally by the component when calling the @content function. :on-[event] - Sets a phx event binding defining the component itself as the default handler (target). This is the prefered way to use phx events in Surface as it can properly handle properties of type :event. Available directives are: :on-phx-click, :on-phx-blur, :on-phx-focus, :on-phx-change, :on-phx-submit, :on-phx-keydown and :on-phx-keyup. Example &lt;div&gt; &lt;div class=&quot;header&quot; :if={{ @showHeader }}&gt; The Header &lt;/div&gt; &lt;ul&gt; &lt;li :for={{ item &lt;- @items }}&gt; {{ item }} &lt;/li&gt; &lt;/ul&gt; &lt;/div&gt;","ref":"Surface.html","title":"Surface","type":"module"},{"doc":"","ref":"Surface.html#component/3","title":"Surface.component/3","type":"function"},{"doc":"Translates Surface code into Phoenix templates.","ref":"Surface.html#sigil_H/2","title":"Surface.sigil_H/2","type":"macro"},{"doc":"Defines a stateless component. Example defmodule Button do use Surface.Component property click, :event def render(assigns) do ~H&quot;&quot;&quot; &lt;button class=&quot;button&quot; phx-click={{ @click }}&gt; {{ @inner_content.() }} &lt;/button&gt; &quot;&quot;&quot; end end Note: Stateless components cannot handle Phoenix LiveView events. If you need to handle them, please use a Surface.LiveComponent instead.","ref":"Surface.Component.html","title":"Surface.Component","type":"behaviour"},{"doc":"This optional callback is invoked in order to set up a context that can be retrieved for any descendent component.","ref":"Surface.Component.html#c:init_context/1","title":"Surface.Component.init_context/1","type":"callback"},{"doc":"A macro component the does not translate any of its contents. You can use this component when you want to skip translation and write regular Phoenix templates code directly or when you don&#39;t want your HTML to be translated at all. Note: By skipping translation, any syntactic sugar provided by Surface will not apply. Properties","ref":"Surface.Components.Raw.html","title":"Surface.Components.Raw","type":"module"},{"doc":"Defines a component that serves as a customizable data holder for the parent component. Example defmodule Column do use Surface.DataComponent property title, :string property field, :string end As you can see, data components don&#39;t need to implement a render/1 callback. Instead, this module defines a data/1 callback that you can use to transform, filter or validate properties before sending them to the parent component. By default all properties are sent as they were originally defined.","ref":"Surface.DataComponent.html","title":"Surface.DataComponent","type":"behaviour"},{"doc":"Implement this callback in order to transform, filter or validate properties before sending them to the parent component.","ref":"Surface.DataComponent.html#c:data/1","title":"Surface.DataComponent.data/1","type":"callback"},{"doc":"This optional callback is invoked in order to set up a context that can be retrieved for any descendent component.","ref":"Surface.DataComponent.html#c:init_context/1","title":"Surface.DataComponent.init_context/1","type":"callback"},{"doc":"A live (stateless or stateful) component. A wrapper around Phoenix.LiveComponent. Example defmodule Dialog do use Surface.LiveComponent property title, :string, required: true def mount(socket) do {:ok, assign(socket, show: false)} end def render(assigns) do ~H&quot;&quot;&quot; &lt;div class={{ &quot;modal&quot;, isActive: @show }}&gt; &lt;div class=&quot;modal-background&quot;&gt;&lt;/div&gt; &lt;div class=&quot;modal-card&quot;&gt; &lt;header class=&quot;modal-card-head&quot;&gt; &lt;p class=&quot;modal-card-title&quot;&gt;{{ @title }}&lt;/p&gt; &lt;/header&gt; &lt;section class=&quot;modal-card-body&quot;&gt; {{ @inner_content.() }} &lt;/section&gt; &lt;footer class=&quot;modal-card-foot&quot; style=&quot;justify-content: flex-end&quot;&gt; &lt;Button click=&quot;hide&quot;&gt;Ok&lt;/Button&gt; &lt;/footer&gt; &lt;/div&gt; &lt;/div&gt; &quot;&quot;&quot; end # Public API def show(dialog_id) do send_update(__MODULE__, id: dialog_id, show: true) end # Event handlers def handle_event(&quot;show&quot;, _, socket) do {:noreply, assign(socket, show: true)} end def handle_event(&quot;hide&quot;, _, socket) do {:noreply, assign(socket, show: false)} end end","ref":"Surface.LiveComponent.html","title":"Surface.LiveComponent","type":"behaviour"},{"doc":"This optional callback is invoked in order to set up a context that can be retrieved for any descendent component.","ref":"Surface.LiveComponent.html#c:init_context/1","title":"Surface.LiveComponent.init_context/1","type":"callback"},{"doc":"A wrapper component around Phoenix.LiveView. Since this module is just a wrapper around Phoenix.LiveView, you cannot define custom properties for it. Only :id and :session are available. However, built-in directives like :for and :if can be used normally. Example defmodule Example do use Surface.LiveView def render(assigns) do ~H&quot;&quot;&quot; &lt;Dialog title=&quot;Alert&quot; id=&quot;dialog&quot;&gt; This &lt;b&gt;Dialog&lt;/b&gt; is a stateful component. Cool! &lt;/Dialog&gt; &lt;Button click=&quot;show_dialog&quot;&gt;Click to open the dialog&lt;/Button&gt; &quot;&quot;&quot; end def handle_event(&quot;show_dialog&quot;, _, socket) do Dialog.show(&quot;dialog&quot;) {:noreply, socket} end end","ref":"Surface.LiveView.html","title":"Surface.LiveView","type":"module"},{"doc":"A low-level component which is responsible for translating its own content at compile time.","ref":"Surface.MacroComponent.html","title":"Surface.MacroComponent","type":"behaviour"},{"doc":"Tranlates the content of the macro component.","ref":"Surface.MacroComponent.html#c:translate/2","title":"Surface.MacroComponent.translate/2","type":"callback"},{"doc":"Defines a behaviour that must be implemented by all HTML/Surface node translators. This module also contains the main logic to translate Surface code.","ref":"Surface.Translator.html","title":"Surface.Translator","type":"behaviour"},{"doc":"","ref":"Surface.Translator.html#c:prepare/2","title":"Surface.Translator.prepare/2","type":"callback"},{"doc":"Translates a string written using the Surface format into a Phoenix template.","ref":"Surface.Translator.html#run/4","title":"Surface.Translator.run/4","type":"function"},{"doc":"Recursively translates nodes from a parsed surface code.","ref":"Surface.Translator.html#translate/2","title":"Surface.Translator.translate/2","type":"function"},{"doc":"","ref":"Surface.Translator.html#c:translate/2","title":"Surface.Translator.translate/2","type":"callback"},{"doc":"","ref":"Surface.Translator.ParseError.html","title":"Surface.Translator.ParseError","type":"exception"}]